package com.azett.mix2stix;

import java.io.File;

import com.azett.mix2stix.config.Configuration;
import com.azett.mix2stix.config.Defaults;
import com.azett.mix2stix.config.Language;
import com.azett.mix2stix.processing.controller.Mix2StixController;
import com.azett.mix2stix.ui.MainWindow;

/**
 * @author azimmermann
 */
public class Mix2Stix {

    /**
     * Startet das Programm.
     * 
     * @author azimmermann
     * @param args
     */
    public static void main(String[] args) {
        // Config und Sprache initialisieren
        Configuration config = Configuration.getConfiguration();
        Language lang = new Language();
        config.setLanguage(lang);

        // TODO: Argumente prfen, ggfs. Hilfetext ausgeben
        // Anzahl Argumente
        switch (args.length) {
            case 0:
                // Aufruf des UIs
                config.setOutputType(Defaults.OUTPUT_TYPE_SWING);
                new MainWindow();
                break;
            case 1:
            case 2:
                printParameterHelp();
                break;
            case 3:
                File src = new File(args[1]);
                File dest = new File(args[2]);
                // Console-Output
                if (args[0].equals("-console")) {
                    config.setOutputType(Defaults.OUTPUT_TYPE_CONSOLE);
                    Mix2StixController processing = new Mix2StixController(src, dest);
                    Thread t = new Thread(processing);
                    t.start();
                }
                // Kein Output
                else if (args[0].equals("-silent")) {
                    config.setOutputType(Defaults.OUTPUT_TYPE_NONE);
                    Mix2StixController processing = new Mix2StixController(src, dest);
                    Thread t = new Thread(processing);
                    t.start();
                }
                // unbekannter Parameter
                else {
                    printParameterHelp();
                }
                break;

            default:
                printParameterHelp();
                break;
        }
    }

    /**
     * Gibt die Parameter auf Konsole aus.
     */
    private static void printParameterHelp() {
        System.err.println(Defaults.PROGRAM_NAME + " " + Defaults.PROGRAM_VERSION);
        System.err.println("Usage: java -jar Mix2Stix.jar [-TYPE SRC DEST]");
        System.err.println();
        System.err.println("Called without arguments, the graphical UI will open. Use the following arguments to supress the GUI:");
        System.err.println("TYPE");
        System.err.println("    silent");
        System.err.println("        - no program output at all");
        System.err.println("    console");
        System.err.println("        - program output on standard console");
        System.err.println();
        System.err.println("SRC");
        System.err.println("    The source directory to process.");
        System.err.println();
        System.err.println("DEST");
        System.err.println("    The destination HTML file to store the result in.");
    }
}
