package com.azett.mix2stix.processing.controller;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import com.azett.mix2stix.config.Configuration;
import com.azett.mix2stix.config.Defaults;
import com.azett.mix2stix.processing.model.Mix2StixModel;
import com.azett.mix2stix.processing.view.AbstractView;
import com.azett.mix2stix.processing.view.ConsoleView;
import com.azett.mix2stix.processing.view.SwingView;

/**
 * Der Controller fr das MVC-Pattern
 * 
 * @author azimmermann
 * 
 */
public class Mix2StixController
        extends AbstractController
        implements Runnable {

    private AbstractView  view;
    private Mix2StixModel model;

    private Configuration config = Configuration.getConfiguration();

    /**
     * Startet den Kopiervorgang
     * 
     * @param srcDir
     * @param destFile
     */
    public Mix2StixController(File srcDir, File destFile) {
        this.model = new Mix2StixModel(srcDir, destFile);
        addModel(this.model);

        // View abhngig vom Output-Type setzen
        switch (config.getOutputType()) {
            // "silent"
            case Defaults.OUTPUT_TYPE_NONE:
                // keine View registrieren
                break;
            // Standard-Console
            case Defaults.OUTPUT_TYPE_CONSOLE:
                this.view = new ConsoleView(this.model);
                addView(this.view);
                break;
            // Swing-UI
            case Defaults.OUTPUT_TYPE_SWING:
                this.view = new SwingView(this.model);
                addView(this.view);
                break;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Runnable#run()
     */
    public void run() {
        // Einlesen starten
        model.process();
    }

    /**
     * ActionListener fr Abbruch
     * 
     * @author azimmermann
     * 
     */
    class CancelListener
            implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            model.cancelProcessing();
        }
    }

}
