package com.azett.mix2stix.ui;

import java.awt.Event;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.URL;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.UIManager;

import MainWindow.MainPanel;
import StdFileFilter;

import com.azett.mix2stix.config.Configuration;
import com.azett.mix2stix.config.Defaults;
import com.azett.mix2stix.config.Language;
import com.azett.mix2stix.processing.controller.Mix2StixController;
import com.azett.mix2stix.util.FileUtils;

/**
 * Programmfenster
 * 
 * @author azimmermann
 */
public class MainWindow
        extends JFrame {

    // Programminfos
    private ProgramInfoWindow programinfowindow = new ProgramInfoWindow();

    // Config und Sprache
    private Configuration     config            = Configuration.getConfiguration();
    private Language          language          = config.getLanguage();

    // Globale Fonts
    final public Font         fontNormal        = new Font("SansSerif", Font.PLAIN, 12);
    final public Font         fontBold          = new Font("SansSerif", Font.BOLD, 12);

    //
    /**
     * Konstruktor
     * 
     * @author azimmermann
     */
    public MainWindow() {
        // System-UI holen
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
        }

        // Men
        Mix2StixMenu menubar = new Mix2StixMenu();
        this.setJMenuBar(menubar);
        menubar.fillMenu();
        this.pack();

        // Oberflche mit Content-Panel aufbauen
        JPanel contentPanel = new ContentPanel();
        this.setContentPane(contentPanel);

        // Fenstergre und -Position berechnen
        Insets myInsets = this.getInsets();
        int myWidth = this.getContentPane().getWidth() + myInsets.left + myInsets.right;
        int myHeight = this.getContentPane().getHeight() + this.getJMenuBar().getHeight() + myInsets.top + myInsets.bottom;
        int myXpos = (Toolkit.getDefaultToolkit().getScreenSize().width - myWidth) / 2; // Fenster horizontal...
        int myYpos = (Toolkit.getDefaultToolkit().getScreenSize().height - myHeight) / 2; // ...und vertikal zentrieren
        this.setBounds(myXpos, myYpos, myWidth, myHeight);

        // sonstige Eigenschaften des Programmfensters
        this.setTitle(Defaults.PROGRAM_NAME);
        this.setResizable(false);

        // das Icon auch innerhalb der JAR finden
        URL imageurl = (this.getClass()).getResource("/com/azett/mix2stix/gfx/icon.gif");
        Image icon = this.getToolkit().getImage(imageurl);
        this.setIconImage(icon);

        this.setVisible(true);
        this.setDefaultCloseOperation(EXIT_ON_CLOSE);
    }

    /**
     * Programm-Men
     * 
     * @author azimmermann
     */
    public class Mix2StixMenu
            extends JMenuBar
            implements ActionListener {
        public JMenu      menuProgram;
        private JMenuItem progCopy;
        private JMenuItem progLoad;
        private JMenuItem progSave;
        private JMenuItem progLog;
        private JMenu     menuLanguage;
        private JMenuItem langLoad;
        private JMenu     menuHelp;
        private JMenuItem helpAbout;

        public Mix2StixMenu() {
            // Programm-Men
            this.menuProgram = new JMenu("");
            this.progCopy = new JMenuItem("");
            progCopy.addActionListener(this);
            KeyStroke kstrCopy = KeyStroke.getKeyStroke(' ', Event.CTRL_MASK);
            progCopy.setAccelerator(kstrCopy);
            menuProgram.add(progCopy);
            menuProgram.addSeparator();
            this.progLoad = new JMenuItem("");
            progLoad.addActionListener(this);
            KeyStroke kstrLoad = KeyStroke.getKeyStroke('O', Event.CTRL_MASK);
            progLoad.setAccelerator(kstrLoad);
            menuProgram.add(progLoad);
            this.progSave = new JMenuItem("");
            progSave.addActionListener(this);
            KeyStroke kstrSave = KeyStroke.getKeyStroke('S', Event.CTRL_MASK);
            progSave.setAccelerator(kstrSave);
            menuProgram.add(progSave);
            menuProgram.addSeparator();
            this.progLog = new JMenuItem("");
            progLog.addActionListener(this);
            KeyStroke kstrLog = KeyStroke.getKeyStroke('L', Event.CTRL_MASK);
            progLog.setAccelerator(kstrLog);
            menuProgram.add(progLog);
            this.add(menuProgram);
            // Sprachmen
            this.menuLanguage = new JMenu("");
            this.langLoad = new JMenuItem("");
            langLoad.addActionListener(this);
            menuLanguage.add(langLoad);
            this.add(menuLanguage);
            // Hilfemen
            this.menuHelp = new JMenu("");
            this.helpAbout = new JMenuItem("");
            helpAbout.addActionListener(this);
            menuHelp.add(helpAbout);
            this.add(menuHelp);
            this.fillMenu();
        }

        // ActionEvent-Auswertung
        public void actionPerformed(ActionEvent e) {
            Object src = new Object();
            src = e.getSource();
            if (src == progCopy) {
                ((MainPanel) getContentPane()).startCopying();
            } else if (src == progLoad) {
                ((MainPanel) getContentPane()).showLoadSettingsDialog();
            } else if (src == progSave) {
                ((MainPanel) getContentPane()).showSaveSettingsDialog();
            } else if (src == progLog) {
                // new LogViewer(logfile, language);
            } else if (src == langLoad) {
                showLoadLanguageDialog();
                // updateLanguage();
                fillMenu();
                ((MainPanel) getContentPane()).fillGui();
            } else if (src == helpAbout) {
                ProgramInfoWindow info = new ProgramInfoWindow();
                info.showProgramInfoWindow();
            }
        }

        public void showLoadLanguageDialog() {
            UiUtils.showLanguageDialog(true);
        }

        public void fillMenu() {
            Graphics g = this.getGraphics();
            this.paint(g);
            this.menuProgram.setText((String) language.getString("menu.program"));
            this.progCopy.setText((String) language.getString("menu.program.copy"));
            this.progLoad.setText((String) language.getString("menu.program.load"));
            this.progSave.setText((String) language.getString("menu.program.save"));
            this.progLog.setText((String) language.getString("menu.program.log"));
            this.menuLanguage.setText((String) language.getString("menu.language"));
            this.langLoad.setText((String) language.getString("menu.language.load"));
            this.menuHelp.setText((String) language.getString("menu.help"));
            this.helpAbout.setText((String) language.getString("menu.help.about"));
            this.repaint();
        }
    }

    /**
     * Content-Panel
     * 
     * @author azimmermann
     */
    private class ContentPanel
            extends JPanel
            implements ActionListener {

        // GUI-Elemente
        JLabel            lblSrcDir;
        JTextField        txtSrcDir;
        JButton           btnSrcDir;
        JLabel            lblDestDir;
        JTextField        txtDestDir;
        JButton           btnDestDir;
        JLabel            lblMaxSize;
        JTextField        txtMaxSize;
        JLabel            lblFilter;
        JScrollPane       scpFilter;
        JTextField        txtFilter;
        JLabel            lblForce;
        JCheckBox         chkClearDest;
        JCheckBox         chkForce;
        JButton           btnCopy;

        private final int TYPE_STANDARD     = 0;
        private final int TYPE_LABEL        = 1;
        private final int TYPE_CHECKBOX     = 2;
        private final int TYPE_OPTIONSPANEL = 3;

        /**
         * Konstruktor
         * 
         * @author azimmermann
         */
        public ContentPanel() {
            setLayout(null);
            lblSrcDir = new JLabel("", JLabel.LEFT);
            lblSrcDir.setFont(fontBold);
            lblSrcDir.setBounds(10, 10, 100, 25);
            add(lblSrcDir);
            txtSrcDir = new JTextField();
            txtSrcDir.setFont(fontNormal);
            txtSrcDir.setBounds(110, 10, 260, 25);
            add(txtSrcDir);
            btnSrcDir = new JButton("...");
            btnSrcDir.setFont(fontBold);
            btnSrcDir.setBounds(370, 10, 20, 25);
            btnSrcDir.addActionListener(this);
            add(btnSrcDir);

            lblDestDir = new JLabel("", JLabel.LEFT);
            lblDestDir.setFont(fontBold);
            lblDestDir.setBounds(10, 40, 100, 25);
            add(lblDestDir);
            txtDestDir = new JTextField();
            txtDestDir.setFont(fontNormal);
            txtDestDir.setBounds(110, 40, 260, 25);
            add(txtDestDir);
            btnDestDir = new JButton("...");
            btnDestDir.setFont(fontBold);
            btnDestDir.setBounds(370, 40, 20, 25);
            btnDestDir.addActionListener(this);
            add(btnDestDir);

            lblMaxSize = new JLabel("", JLabel.LEFT);
            lblMaxSize.setFont(fontBold);
            lblMaxSize.setBounds(10, 70, 100, 25);
            add(lblMaxSize);
            txtMaxSize = new JTextField();
            txtMaxSize.setFont(fontNormal);
            txtMaxSize.setBounds(110, 70, 80, 25);
            add(txtMaxSize);

            lblFilter = new JLabel("", JLabel.LEFT);
            lblFilter.setFont(fontBold);
            lblFilter.setBounds(10, 100, 100, 25);
            add(lblFilter);
            txtFilter = new JTextField("");
            txtFilter.setFont(fontNormal);
            txtFilter.setBounds(110, 100, 80, 25);
            add(txtFilter);

            chkClearDest = new JCheckBox("");
            chkClearDest.setFont(fontBold);
            chkClearDest.setBounds(200, 70, 200, 25);
            chkClearDest.addActionListener(this);
            add(chkClearDest);

            chkForce = new JCheckBox("");
            chkForce.setFont(fontBold);
            chkForce.setBounds(200, 100, 200, 25);
            chkForce.addActionListener(this);
            add(chkForce);

            btnCopy = new JButton("");
            btnCopy.setFont(fontBold);
            btnCopy.setBounds(10, 140, 380, 30);
            btnCopy.setToolTipText("");
            btnCopy.addActionListener(this);
            add(btnCopy);

            // Initial-Einstellungen aus Datei laden
            loadSettingsFromFile(myFunctionClass.getLatestSettingsFileFromConf());

            // GUI mit Sprache fllen
            fillGui();

            this.setSize(400, 180);
            this.setVisible(true);
        }

        /**
         * Spracheinstellungen laden und auf die GUI bringen
         * 
         * @author azimmermann
         */
        public void fillGui() {
            Graphics g = this.getGraphics();
            this.paint(g);
            this.lblSrcDir.setText((String) language.getString("msg.sources") + ": ");
            this.lblDestDir.setText((String) language.getString("msg.destination") + ": ");
            this.lblMaxSize.setText((String) language.getString("msg.maxmb") + ": ");
            this.lblFilter.setText((String) language.getString("msg.filter") + ": ");
            this.chkClearDest.setText((String) language.getString("msg.cleardestination"));
            this.chkForce.setText((String) language.getString("msg.forceoverwrite"));
            this.btnCopy.setText((String) language.getString("msg.startcopying"));
            this.repaint();
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent e) {
            Object src = new Object();
            src = e.getSource();
            // Button "Kopieren"
            if (src == btnCopy) {
                Mix2StixController processing = new Mix2StixController(srcdir, destfile);
                Thread t = new Thread(processing);
                t.start();
            }
            // Checkbox "Zielverzeichnis leeren"
            else if (src == chkClearDest) {
                if (chkClearDest.isSelected()) {
                    showWarningDialog((String) language.get("warningcleardestination"));
                    chkForce.setSelected(false);
                    chkForce.setEnabled(false);
                } else
                    chkForce.setEnabled(true);
            }
            // Checkbox "Vorhandene Dateien ersetzen"
            else if (src == chkForce) {
                if (chkForce.isSelected()) {
                    showWarningDialog((String) language.get("warningforceoverwrite"));
                }
            }
            // Button "Quellverzeichnis auswhlen"
            else if (src == btnSrcDir) {
                String currentInput = txtSrcDir.getText();
                String lastInput = "";
                JFileChooser chooser;
                // zunchst schauen, was schon im Textfeld steht
                StringTokenizer t = new StringTokenizer(currentInput, ";");
                if (t.hasMoreTokens()) {
                    // dem FileChooser das Parent-Verzeichnis des letzten Eintrags des Textfelds mitgeben
                    while (t.hasMoreTokens())
                        lastInput = t.nextToken();
                    File temp = new File(lastInput);
                    // wenn lastInput nicht schon ein Wurzelverz. ist: den Chooser mit lastInputs Wurzelverz. aufrufen
                    if (!(temp.getParent() == null))
                        lastInput = temp.getParent();
                    chooser = new JFileChooser(lastInput);
                }
                // ansonsten FileChooser ohne Vorgabe ffnen
                else
                    chooser = new JFileChooser();
                chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                chooser.setApproveButtonText((String) language.get("choosesourcebuttonchoose"));
                chooser.setApproveButtonToolTipText((String) language.get("choosesourcebuttonchoosetooltip"));
                chooser.setDialogTitle((String) language.get("choosesourcetitle"));
                chooser.setMultiSelectionEnabled(true);
                int returnVal = chooser.showOpenDialog(this);
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    // alle gewhlten Verz. mit ; trennen
                    String newInput = "";
                    for (int i = 0; i < chooser.getSelectedFiles().length; i++) {
                        if (i == chooser.getSelectedFiles().length - 1)
                            newInput = newInput + chooser.getSelectedFiles()[i].getPath();
                        else
                            newInput = newInput + chooser.getSelectedFiles()[i].getPath() + ";";
                    }
                    // wenn schon etwas im Textfeld stand: mit ; anhngen
                    if (!currentInput.equals(""))
                        txtSrcDir.setText(currentInput + ";" + newInput);
                    // sonst einfach ins Textfeld schreiben
                    else
                        txtSrcDir.setText(newInput);
                }
            }
            // Button "Zielverzeichnis auswhlen"
            else if (src == btnDestDir) {
                String currentInput = txtDestDir.getText();
                File temp = new File(currentInput);
                // wenn currentInput nicht schon ein Wurzelverz. ist: den Chooser mit currentInputs Wurzelverz. aufrufen
                if (!(temp.getParent() == null))
                    currentInput = temp.getParent();
                JFileChooser chooser = new JFileChooser(currentInput);
                chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                chooser.setApproveButtonText((String) language.get("choosedestinationbuttonchoose"));
                chooser.setApproveButtonToolTipText((String) language.get("choosedestinationbuttonchoosetooltip"));
                chooser.setDialogTitle((String) language.get("choosedestinationtitle"));
                int returnVal = chooser.showOpenDialog(this);
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    txtDestDir.setText(chooser.getSelectedFile().getPath());
                }
            }
        }

        /**
         * Zeigt den Auswahldialog fr das einzulesende Verzeichnis und gibt selbiges zurck.
         * 
         * @author azimmermann
         * @return das gewhlte Verzeichnis
         */
        private File getSourceDir() {
            JFileChooser chooser = new JFileChooser();
            chooser.setDialogType(JFileChooser.OPEN_DIALOG);
            chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            chooser.setDialogTitle(language.getString("msg.selectDirectoryToRead"));
            return (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) ? chooser.getSelectedFile() : null;
        }

        /**
         * Zeigt den Auswahldialog fr die zu speichernde Datei gibt selbige zurck.
         * 
         * @author azimmermann
         * @param sourceDir
         *            das einzulesende Verzeichnis
         * @return die gewhlte Datei
         */
        private File getDestinationFile(File sourceDir) {
            JFileChooser chooser = new JFileChooser();
            chooser.setDialogType(JFileChooser.SAVE_DIALOG);
            chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
            chooser.setDialogTitle(language.getString("msg.saveHtmlAs"));
            String[] filters = { "htm", "html" };
            chooser.addChoosableFileFilter(new StdFileFilter(filters, language.getString("msg.htmlFiles")));

            chooser.setSelectedFile(new File(Defaults.PROGRAM_NAME + "_" //
                    + FileUtils.cleanFileName(sourceDir.getName()) //
                    + ".html"));
            // Gltige Datei im Chooser selektiert
            if ((chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
            // Datei noch nicht vorhanden ODER berschreiben besttigt
                    && ((!chooser.getSelectedFile().exists()) || (JOptionPane.showConfirmDialog(this,
                            language.getString("msg.overWriteFile", new String[] { chooser.getSelectedFile().getName() }),
                            language.getString("msg.confirmation"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)))
                return chooser.getSelectedFile();
            else
                return null;
        }

        /**
         * Setzt die Einstellungen im GUI gem der gespeicherten Konfiguration.
         * 
         * @author azimmermann
         */
        public void setOptionsInGui() {
            // // Optionen setzen
            // chkMultiDocument.setSelected(config.getBoolean(Configuration.CREATE_MULTI_DOCUMENT));
            // chkHidden.setSelected(config.getBoolean(Configuration.DISPLAY_HIDDEN));
            // chkFullPath.setSelected(config.getBoolean(Configuration.DISPLAY_FULL_PATH));
            // chkTooltip.setSelected(config.getBoolean(Configuration.DISPLAY_TOOLTIP));
            // chkInfotable.setSelected(config.getBoolean(Configuration.DISPLAY_PROGRAM_INFO));
            // chkSize.setSelected(config.getBoolean(Configuration.DISPLAY_FILE_SIZE));
            // chkLastChange.setSelected(config.getBoolean(Configuration.DISPLAY_LAST_CHANGE));
            // chkBitrate.setSelected(config.getBoolean(Configuration.DISPLAY_MP3_BITRATE));
            // chkLinks.setSelected(config.getBoolean(Configuration.CREATE_LINKS));
            // chkLinksAsTargetBlank.setSelected(config.getBoolean(Configuration.CREATE_LINKS_AS_TARGET_BLANK));
            // if (!this.chkLinks.isSelected()) {
            // this.chkLinksAsTargetBlank.setSelected(false);
            // this.chkLinksAsTargetBlank.setEnabled(false);
            // }
        }

        /**
         * Speichert die aktuell im GUI eingestellten Optionen in die Konfigurationsdatei.
         * 
         * @author azimmermann
         */
        public void saveSettingsToFile() {
            // config.setBoolean(Configuration.CREATE_MULTI_DOCUMENT, chkMultiDocument.isSelected());
            // config.setBoolean(Configuration.DISPLAY_HIDDEN, chkHidden.isSelected());
            // config.setBoolean(Configuration.DISPLAY_FULL_PATH, chkFullPath.isSelected());
            // config.setBoolean(Configuration.DISPLAY_TOOLTIP, chkTooltip.isSelected());
            // config.setBoolean(Configuration.DISPLAY_PROGRAM_INFO, chkInfotable.isSelected());
            // config.setBoolean(Configuration.DISPLAY_FILE_SIZE, chkSize.isSelected());
            // config.setBoolean(Configuration.DISPLAY_LAST_CHANGE, chkLastChange.isSelected());
            // config.setBoolean(Configuration.DISPLAY_MP3_BITRATE, chkBitrate.isSelected());
            // config.setBoolean(Configuration.CREATE_LINKS, chkLinks.isSelected());
            // config.setBoolean(Configuration.CREATE_LINKS_AS_TARGET_BLANK, chkLinksAsTargetBlank.isSelected());
            // config.saveToFile();
        }

        /**
         * Erstellt neue {@link GridBagConstraints} abhngig vom bergebenen Element-Typ.
         * 
         * @author azimmermann
         * @param type
         *            Der Typ
         * @param x
         *            X-Position
         * @param y
         *            Y-Position
         * @param width
         *            Breite
         * @param height
         *            Hhe
         * @return die fertigen {@link GridBagConstraints}
         */
        private GridBagConstraints makegbc(int type, int x, int y, int width, int height) {
            GridBagConstraints gbc = new GridBagConstraints();
            gbc.gridx = x;
            gbc.gridy = y;
            gbc.gridwidth = width;
            gbc.gridheight = height;
            // Labels
            if (type == TYPE_LABEL) {
                gbc.insets = new Insets(7, 2, 2, 2);
                gbc.anchor = GridBagConstraints.WEST;
            }
            // Checkboxes
            else if (type == TYPE_CHECKBOX) {
                gbc.insets = new Insets(0, 5, 0, 2);
                gbc.anchor = GridBagConstraints.WEST;
            }
            // Options-Panel
            else if (type == TYPE_OPTIONSPANEL) {
                gbc.insets = new Insets(10, 2, 10, 2);
            }
            // Standard
            else {
                gbc.insets = new Insets(0, 2, 0, 2);
            }
            return gbc;
        }
    }
}
