package com.azett.mix2stix.ui;

import java.awt.Component;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Properties;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import StdFileFilter;

import com.azett.mix2stix.config.Configuration;
import com.azett.mix2stix.config.Defaults;
import com.azett.mix2stix.util.FileComparator;

/**
 * @author azimmermann
 */
public class UiUtils {

    /**
     * Zeigt den Sprachauswahl-Dialog und gibt die gewhlte Sprache zurck.
     * 
     * @author azimmermann
     * @return
     */
    public static String showLanguageDialog(boolean writeToConfig) {
        // System-UI holen
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
        }

        // Vorhandene Sprachdateien einlesen
        HashMap langMap = new HashMap();
        Properties currentLang = new Properties();

        File[] fileList = Defaults.LANGUAGE_DIR.listFiles();
        Arrays.sort(fileList, new FileComparator());
        Object[] possibilities = new Object[fileList.length];
        for (int i = 0; i < fileList.length; i++) {
            try {
                currentLang.load(new FileInputStream(fileList[i]));
            } catch (IOException e) {
                showErrorDialog("Error!", "Error reading file " + fileList[i]);
                System.exit(0);
            }
            langMap.put(currentLang.get("lang.name"), fileList[i].getName().substring(0, fileList[i].getName().lastIndexOf('.')));
            possibilities[i] = currentLang.get("lang.name");
        }

        String s = (String) JOptionPane.showInputDialog(null, "<html><b>Welcome to " + Defaults.PROGRAM_NAME
                + "!</b></html>\n\nPlease select your language:", Defaults.PROGRAM_NAME, JOptionPane.QUESTION_MESSAGE, null,
                possibilities, possibilities[0]);

        // If a string was returned, say so.
        if ((s != null) && (s.length() > 0)) {
            if (writeToConfig) {
                Configuration.getConfiguration().setString("language", langMap.get(s).toString());
                Configuration.getConfiguration().saveToFile();
            }
            return langMap.get(s).toString();
        }

        else {
            System.exit(0);
            return "";
        }
    }

    /**
     * Zeigt einen Fehlerdialog an.
     * 
     * @author azimmermann
     * @param title
     * @param msg
     */
    public static void showErrorDialog(String title, String msg) {
        showErrorDialog(null, title, msg);
    }

    /**
     * Zeigt einen Fehlerdialog auf dem bergebenen Parent-{@link Component} an.
     * 
     * @author azimmermann
     * @param parent
     * @param title
     * @param msg
     */
    public static void showErrorDialog(Component parent, String title, String msg) {
        JOptionPane.showMessageDialog(parent, msg, title, JOptionPane.ERROR_MESSAGE);
    }

    /**
     * @author azimmermann
     */
    public void showSaveSettingsDialog() {
        if (!inputIsCorrect()) {
            return;
        }
        String[] inputs = { this.txtSrcDir.getText(), this.txtDestDir.getText(), this.txtMaxSize.getText(),
                this.txtFilter.getText(), String.valueOf(this.chkClearDest.isSelected()),
                String.valueOf(this.chkForce.isSelected()) };
        JFileChooser chooser = new JFileChooser(".");
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        chooser.addChoosableFileFilter(new StdFileFilter(settingsext, (String) language.get("settingstypedescription")));
        chooser.setAcceptAllFileFilterUsed(false);
        chooser.setApproveButtonText((String) language.get("savesettingsbuttonsave"));
        chooser.setApproveButtonToolTipText((String) language.get("savesettingsbuttonsavetooltip"));
        chooser.setDialogTitle((String) language.get("savesettingstitle"));
        int returnVal = chooser.showSaveDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            String fileName = chooser.getSelectedFile().getName();
            // auf Standard-Settings-Dateiendung (settingsext) prfen, evtl. anhngen
            File settingsFile;
            if (!(fileName.length() > 4)
                    || !(fileName.substring(fileName.length() - 4, fileName.length()).equals("." + settingsext)))
                settingsFile = new File(chooser.getSelectedFile().getParent(), fileName + "." + settingsext);
            else
                settingsFile = chooser.getSelectedFile();

            myFunctionClass.saveSettingsToFile(inputs, settingsFile);
        }
    }

    /**
     * "Einstellungen laden"-Dialg anzeigen
     * 
     * @author azimmermann
     */
    public void showLoadSettingsDialog() {
        JFileChooser chooser = new JFileChooser(".");
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        chooser.addChoosableFileFilter(new StdFileFilter(settingsext, (String) language.get("settingstypedescription")));
        chooser.setAcceptAllFileFilterUsed(false);
        chooser.setApproveButtonText((String) language.get("loadsettingsbuttonload"));
        chooser.setApproveButtonToolTipText((String) language.get("loadsettingsbuttonloadtooltip"));
        chooser.setDialogTitle((String) language.get("loadsettingstitle"));
        int returnVal = chooser.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            loadSettingsFromFile(chooser.getSelectedFile());
        }
    }

    /**
     * "Einstellungen speichern"-Dialog anzeigen
     * 
     * @author azimmermann
     */
    public void showSaveSettingsDialog() {
        if (!inputIsCorrect()) {
            return;
        }
        String[] inputs = { this.txtSrcDir.getText(), this.txtDestDir.getText(), this.txtMaxSize.getText(),
                this.txtFilter.getText(), String.valueOf(this.chkClearDest.isSelected()),
                String.valueOf(this.chkForce.isSelected()) };
        JFileChooser chooser = new JFileChooser(".");
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        chooser.addChoosableFileFilter(new StdFileFilter(settingsext, (String) language.get("settingstypedescription")));
        chooser.setAcceptAllFileFilterUsed(false);
        chooser.setApproveButtonText((String) language.get("savesettingsbuttonsave"));
        chooser.setApproveButtonToolTipText((String) language.get("savesettingsbuttonsavetooltip"));
        chooser.setDialogTitle((String) language.get("savesettingstitle"));
        int returnVal = chooser.showSaveDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            String fileName = chooser.getSelectedFile().getName();
            // auf Standard-Settings-Dateiendung (settingsext) prfen, evtl. anhngen
            File settingsFile;
            if (!(fileName.length() > 4)
                    || !(fileName.substring(fileName.length() - 4, fileName.length()).equals("." + settingsext)))
                settingsFile = new File(chooser.getSelectedFile().getParent(), fileName + "." + settingsext);
            else
                settingsFile = chooser.getSelectedFile();

            myFunctionClass.saveSettingsToFile(inputs, settingsFile);
        }
    }
}
