package com.azett.mix2stix.util;

import java.io.BufferedInputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;

import com.azett.mix2stix.ui.UiUtils;

public class FileUtils {

    /**
     * Schreibt den bergebenen String UTF8-kodiert in die bergebene Datei.
     * 
     * @param content
     * @param file
     * @return
     */
    public static boolean writeContentToFile(String content, File file) {
        try {
            Writer out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), "UTF-8"));
            out.write(content);
            out.close();
            return true;
        } catch (IOException ioex) {
            return false;
        }
    }

    /**
     * Gibt den Inhalt der bergebenen Datei als String zurck.
     * 
     * @author azimmermann
     * @param file
     * @return
     */
    public static String getFileContent(File file) {
        byte[] buffer = new byte[(int) file.length()];
        BufferedInputStream f = null;
        try {
            f = new BufferedInputStream(new FileInputStream(file));
            f.read(buffer);
        } catch (IOException e) {
            UiUtils.showErrorDialog("Error while reading " + file.getAbsolutePath(), e.getMessage());
        } finally {
            if (f != null)
                try {
                    f.close();
                } catch (IOException ignored) {
                }
        }
        return new String(buffer);
    }

    /**
     * Bereitet einen Dateipfad fr Verlinkung auf.
     * 
     * @author azimmermann
     * @param filePath
     * @return
     */
    public static String cleanFilePathForLink(String filePath) {
        return filePath.replace('\\', '/');
        // .replace(" ", "%20"); // checkt der bekloppte IE nicht -.-
    }

    /**
     * Bereinigt den bergebenen Namen, um das Anlegen einer Datei mit diesem Namen zu ermglichen.
     * 
     * @author azimmermann
     * @param name
     *            der zu bereinigende Name
     * @return der bereinigte Name
     */
    public static String cleanFileName(String name) {
        // Aktuell auf Windows spezialisiert, wo folgende Zeichen in Dateinamen
        // nicht erlaubt sind: /\*:?"<>|
        return name.replaceAll("[\\/\\\\\\*\\:\\?\\\"\\<\\>|]", "");
    }
}
