package com.infosys.propertyeditor;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;



public class PropEditor  {
    
    public static EditorFile eFile;
    
    public static void editFile(Frame f) {
        JFileChooser fc = new JFileChooser(new File("translations"));
        fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fc.setDialogTitle("Choose a language file...");
        
        if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
        	openFile(f, fc.getSelectedFile());
        }
        else {
        	//System.exit(0);
        } 
    }
    
    public static void openFile(Frame f, File selectedFile){
		eFile = new EditorFile(selectedFile.getPath());
            
		eFile.init();
            
		int n = Parser.parsePropertyFile(eFile);
            
		if(n > 0){
			PropertyEditorDialog ed = new PropertyEditorDialog(f, eFile);
			ed.setVisible(true);
		}
		else {
			JOptionPane.showMessageDialog(null,
			"Could not parse property file:\n" + selectedFile.getAbsolutePath() + 
			".\nPlease contact IPSP Support.",
			"Error in Property File",
			JOptionPane.ERROR_MESSAGE);
		}
    }
    
    public static int closeFile(){
/*    	
        if(eFile.isModified()) {
            Object[] options = {"Save", "Don't Save", "Cancel"};
            
            int n = JOptionPane.showOptionDialog(null,
            "Are you sure you want to close the file without saving?", "Warning",
            JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE,
            null, options, options[0]
            );
            
            return n;
        }
*/        
        return 1;
    }
    
    public static void saveFile() {
        if(eFile.exists() && eFile.isModified()){
            try{
				String newLine = System.getProperty("line.separator");
                BufferedWriter bw = new BufferedWriter(new FileWriter(eFile));
                System.out.println(eFile.numGroup());
                for (int i=0; i < eFile.numGroup(); i++) {
                    bw.write(eFile.groupAt(i).toText() + newLine);
                }
                bw.close();
                
                eFile.setModified(false);
            } 
            catch(Exception e) {
            	//System.out.println("Error when saving");
				JOptionPane.showMessageDialog(null,
				"Could not save properties:\n" + eFile.getAbsolutePath() + 
				".\nPlease contact IPSP Support.",
				"Error in Saving Properties",
				JOptionPane.ERROR_MESSAGE);
            }
        }
    }
    
    public static void main(String args[]) throws Exception{
        
        
        //UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName()); 
        //JFrame.setDefaultLookAndFeelDecorated(true);

    	JFrame frame = new JFrame("Mix2Stix EditLanguage");

        
        frame.setSize(400,300);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
        frame.setLocation((d.width-frame.getWidth())/2, (d.height-frame.getHeight())/2);
                
        JMenuBar menuBar = new JMenuBar();
        frame.setJMenuBar(menuBar);
		
		JMenu projectMenu = new JMenu("Program");
		menuBar.add(projectMenu);

		JMenuItem loadProject = new JMenuItem("Load language file");
		projectMenu.add(loadProject);

		JMenu projectHelp = new JMenu("?");
		menuBar.add(projectHelp);

		JMenuItem aboutUsage = new JMenuItem("Usage");
		projectHelp.add(aboutUsage);

		aboutUsage.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent arg0) {
		    	JOptionPane.showMessageDialog(null, 
		    			"=== Usage of EditLanguage ===\n" +
		    			"\n" +
		    			"EditLanguage allows you to change the translation\n" +
		    			"of Mix2Stix by editing the language properties file.\n" +
		    			"Of course, you can also do so with your familiar\n" +
		    			"text editor, but that wouldn't be that comfortable ;)\n" +
		    			"\n" +
		    			"To edit a language file, open it by clicking 'Program',\n" +
		    			"'Load language file' from '[Mix2Stix dir]/translations'\n" +
		    			"and set the values to your language.\n" +
		    			"After editing the translation, the file is saved by\n" +
		    			"pressing the 'OK' button. To preserve existing\n" +
		    			"translations, make a copy before editing a file.\n" +
		    			"\n" +
		    			"If your language is not listed at www.mix2stix.de.ms,\n" +
		    			"feel free to create a new translation by editing an\n" +
		    			"existing one, and send it to mix2stix@azett.com.");
			}
			
		}
        );
		
		JMenuItem aboutProgram = new JMenuItem("About");
		projectHelp.add(aboutProgram);

		aboutProgram.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent arg0) {
		    	JOptionPane.showMessageDialog(null, 
		    			"=== About EditLanguage ===\n" +
		    			"\n" +
		    			"Slightly changed PropEditor from the Properties\n" +
		    			"Editor Framework by David Bismut, Krishnakumar\n" +
		    			"Pooloth and Swaminathan Natarajan. Get more\n" +
		    			"information at java.sys-con.com/read/99786.htm.\n" +
		    			"\n" +
		    			"Edited for the Mix2Stix project by azett 2005."); 
			}
			
		}
        );
		
		loadProject.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				
				editFile(null);
			}
			}
        );
        
		JMenuItem exit = new JMenuItem("Exit");
		projectMenu.add(exit);
		exit.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent arg0) {
				System.exit(0);
				
			}
			
		}
        );
        
        
        frame.getContentPane().setLayout(new BorderLayout());
        frame.setVisible(true);

    }
    
}